unit DCommon;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBClient, MConnect, SConnect, ActnList;

type
  TDMCommon = class(TDataModule)
    scCommon: TSocketConnection;
    cdsDocList: TClientDataSet;
    cdsClient: TClientDataSet;
    dsDocList: TDataSource;
    dsClient: TDataSource;
    ActionList: TActionList;
    actNewDoc: TAction;
    actEditDoc: TAction;
    actDelDoc: TAction;
    actShowClients: TAction;
    procedure cdsClientNewRecord(DataSet: TDataSet);
    procedure cdsClientReconcileError(DataSet: TClientDataSet;
      E: EReconcileError; UpdateKind: TUpdateKind;
      var Action: TReconcileAction);
    procedure actShowClientsExecute(Sender: TObject);
    procedure actDelDocExecute(Sender: TObject);
    procedure cdsDocListReconcileError(DataSet: TClientDataSet;
      E: EReconcileError; UpdateKind: TUpdateKind;
      var Action: TReconcileAction);
    procedure cdsDocListBeforeEdit(DataSet: TDataSet);
    procedure cdsDocListBeforeInsert(DataSet: TDataSet);
    procedure actEditDocExecute(Sender: TObject);
    procedure actNewDocExecute(Sender: TObject);
    procedure cdsDocListBeforeDelete(DataSet: TDataSet);
  private
    function GetClientName(ID: integer): string;
    { Private declarations }
  public
    function ShowClients: Integer;
    property ClientName[ID: integer]: string read GetClientName;
    { Public declarations }
  end;

var
  DMCommon: TDMCommon;

implementation

uses DocServer_TLB, FClient, DDoc;

{$R *.DFM}

procedure TDMCommon.cdsClientNewRecord(DataSet: TDataSet);
var
  AServer: IrdmCommonDisp;
begin
  AServer := IrdmCommonDisp(scCommon.GetServer);
  cdsClient.FieldByName('CLIENT_ID').AsInteger := AServer.NewClientID;
  AServer := nil;
end;

function TDMCommon.GetClientName(ID: integer): string;
var
  AServer: IrdmCommonDisp;
begin
  Result := '';
  if ID = 0 then Exit;
  AServer := IrdmCommonDisp(scCommon.GetServer);
  Result := AServer.ClientName[ID];
  AServer := nil;
end;

function TDMCommon.ShowClients: Integer;
begin
  Result := 0;
  Application.CreateForm(TFormClients, FormClients);
  try
    if FormClients.ShowModal = mrOK then
      Result := FormClients.ClientID;
  finally
    FormClients.Release;
    FormClients := nil;
  end;
end;

procedure TDMCommon.cdsClientReconcileError(DataSet: TClientDataSet;
  E: EReconcileError; UpdateKind: TUpdateKind;
  var Action: TReconcileAction);
begin
  Action := raCancel;
  ShowMessage(E.Message);
end;

procedure TDMCommon.actShowClientsExecute(Sender: TObject);
begin
  ShowClients;
end;

procedure TDMCommon.actDelDocExecute(Sender: TObject);
begin
  with cdsDocList do
  begin
    Delete;
    ApplyUpdates(0);
  end;
end;

procedure TDMCommon.cdsDocListReconcileError(DataSet: TClientDataSet;
  E: EReconcileError; UpdateKind: TUpdateKind;
  var Action: TReconcileAction);
begin
  Action := raCancel;
  ShowMessage(E.Message);
end;

procedure TDMCommon.cdsDocListBeforeEdit(DataSet: TDataSet);
begin
  Abort;
end;

procedure TDMCommon.cdsDocListBeforeInsert(DataSet: TDataSet);
begin
  Abort;
end;

procedure TDMCommon.actEditDocExecute(Sender: TObject);
var
  DocID: Integer;
begin
  DocID := cdsDocList.FieldByName('DOC_ID').AsInteger;
  if DocID <= 0 then
    raise Exception.Create('  ');
  if DMDoc.ProcessDoc(DocID, False) then
  begin
    cdsDocList.Active := False;
    cdsDocList.Active := True;
  end;
end;

procedure TDMCommon.actNewDocExecute(Sender: TObject);
begin
  if DMDoc.ProcessDoc(0, True) then
  begin
    cdsDocList.Active := False;
    cdsDocList.Active := True;
  end;
end;

procedure TDMCommon.cdsDocListBeforeDelete(DataSet: TDataSet);
begin
  if MessageDlg('    ?',
    mtConfirmation,[mbYes, mbNo],0) <> mrYes
  then
    Abort;
end;

end.
