unit DDoc;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBClient, MConnect, SConnect;

type
  TDMDoc = class(TDataModule)
    scDoc: TSocketConnection;
    cdsTitle: TClientDataSet;
    cdsTitleDOC_NUM: TStringField;
    cdsTitleDOC_DATE: TDateTimeField;
    cdsTitleFROM_ID: TIntegerField;
    cdsTitleTO_ID: TIntegerField;
    cdsBody: TClientDataSet;
    cdsBodyLINE_NUM: TIntegerField;
    cdsBodyCONTENT: TStringField;
    dsTitle: TDataSource;
    dsBody: TDataSource;
    cdsTitleSumma: TCurrencyField;
    cdsTitleToName: TStringField;
    cdsTitleFromName: TStringField;
    cdsBodyRowSum: TCurrencyField;
    cdsBodyCOUNT_NUM: TCurrencyField;
    cdsBodyPRICE: TCurrencyField;
    procedure cdsBodyCalcFields(DataSet: TDataSet);
    procedure cdsBodyAfterPost(DataSet: TDataSet);
    procedure cdsBodyLINE_NUMValidate(Sender: TField);
  private
    function GetDocSum: Currency;
    procedure RecalcDocSum;
    { Private declarations }
  public
    function ProcessDoc(DocID: Integer; NewDoc: Boolean): boolean;
    function ShowEditForm: Boolean;
    function ApplyDoc: boolean; //  
    procedure SetSenderName(Value: integer);
    procedure SetReceiverName(Value: integer);
    property DocSum: Currency read GetDocSum;
    { Public declarations }
  end;

var
  DMDoc: TDMDoc;

implementation

uses DocServer_TLB, FDoc, DCommon;

{$R *.DFM}

{ TDMDoc }

function TDMDoc.ApplyDoc: boolean;
var
  AServer: IrdmDocDisp;
  ErrorLog: string;
begin
  with cdsTitle do
  begin
    if State in [dsEdit, dsInsert] then
      Post;
    if ChangeCount > 0 then
      ApplyUpdates(0);
  end;
  with cdsBody do
  begin
    if State in [dsEdit, dsInsert] then
      Post;
    if ChangeCount > 0 then
      ApplyUpdates(-1);
  end;
  AServer := IrdmDocDisp(scDoc.GetServer);
  ErrorLog := AServer.ApplyChanges;
  if ErrorLog <> '' then
  begin
    MessageDlg('  :'#13#10 + ErrorLog,
      mtError, [mbOK], 0);
    //
    cdsBody.Active := False;
    cdsTitle.Active := False;
    cdsTitle.Active := True;
    cdsBody.Active := True;
  end;
  Result := ErrorLog = '';
end;

function TDMDoc.ProcessDoc(DocID: Integer; NewDoc: Boolean): boolean;
var
  AServer: IrdmDocDisp;
begin
  AServer := IrdmDocDisp(scDoc.GetServer);
  if NewDoc then
    AServer.CreateNewDoc
  else
    AServer.DocID := DocID;
  try
    cdsTitle.Active := True;
    cdsBody.Active := True;
    RecalcDocSum;
    Result := ShowEditForm;
    cdsTitle.Active := false;
    cdsBody.Active := false;
  finally
    AServer.DocID := 0; //unregister document
  end;
end;

function TDMDoc.ShowEditForm: Boolean;
begin
  Application.CreateForm(TFormDoc, FormDoc);
  try
    Result := FormDoc.ShowModal = mrOK;
  finally
    FormDoc.Release;
    FormDoc := nil;
  end;
end;

procedure TDMDoc.cdsBodyCalcFields(DataSet: TDataSet);
begin
  if cdsBody.state = dsInternalCalc then
    cdsBodyRowSum.asCurrency :=
      cdsBodyCount_Num.asCurrency * cdsBodyPRICE.asCurrency;
end;

procedure TDMDoc.SetReceiverName(Value: integer);
begin
  with DMDoc.cdsTitle do
  begin
    if not (State in [dsEdit, dsInsert]) then
      Edit;
    if Value <> 0 then
      FieldByName('TO_ID').AsInteger := Value
    else
      FieldByName('TO_ID').Clear;
    FieldByName('ToName').AsString :=
      DMCommon.ClientName[FieldByName('TO_ID').AsInteger];
  end;
end;

procedure TDMDoc.SetSenderName(Value: integer);
begin
  with cdsTitle do
  begin
    if not (State in [dsEdit, dsInsert]) then
      Edit;
    if Value <> 0 then
      FieldByName('FROM_ID').AsInteger := Value
    else
      FieldByName('FROM_ID').Clear;
    FieldByName('FromName').AsString :=
      DMCommon.ClientName[FieldByName('FROM_ID').AsInteger];
    //    
  end;
end;

procedure TDMDoc.cdsBodyAfterPost(DataSet: TDataSet);
begin
  RecalcDocSum;
end;

procedure TDMDoc.RecalcDocSum;
begin
  with cdsBody do
    if ChangeCount > 0 then
      ApplyUpdates(-1);
  with cdsTitle do
  begin
    if not (State in [dsEdit, dsInsert]) then
      Edit;
    FieldByName('Summa').asCurrency := GetDocSum;
  end;
end;

function TDMDoc.GetDocSum: Currency;
var
  AServer: IrdmDocDisp;
begin
  AServer := IrdmDocDisp(scDoc.GetServer);
  Result := AServer.DocSum;
end;

procedure TDMDoc.cdsBodyLINE_NUMValidate(Sender: TField);
begin
  if Sender.AsInteger < 0 then
    raise Exception.Create('  , ');
end;

end.
